uses Windows;

const
  Version = '0.2';

  SPI_GETWINARRANGING = $82;
  SPI_SETWINARRANGING = $83;

procedure CheckError(R: BOOL);
var
  buf: PChar;
begin
  if R then
    Exit;
  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM or FORMAT_MESSAGE_ALLOCATE_BUFFER
      or FORMAT_MESSAGE_IGNORE_INSERTS or $FF, nil, GetLastError,
      (SUBLANG_DEFAULT shl 10) or LANG_NEUTRAL, PChar(@buf), 0, nil);
  MessageBox(GetDesktopWindow, buf, 'xmouse', MB_ICONERROR);
  LocalFree(HLOCAL(buf));
  Halt(1);
end;

function IsWinArrSupported: boolean;
var
  osvi: TOSVersionInfo;
begin
  FillChar(osvi, SizeOf(osvi), 0);
  osvi.dwOSVersionInfoSize := SizeOf(osvi);
  GetVersionEx(osvi);
  Result := (osvi.dwMajorVersion >= 6) and (osvi.dwMinorVersion >= 1);
end;

procedure Usage;
  function BStr(Value: Boolean): string;
  begin
    if Value then
      Result := 'on'
    else
      Result := 'off';
  end;
  function IStr(Value: Integer): string;
  begin
    Str(Value, Result);
  end;
var
  timeout: Integer;
  tracking: BOOL;
  zorder: BOOL;
  winarr: BOOL;
  winarrs: string;
begin
  CheckError(SystemParametersInfo(SPI_GETACTIVEWINDOWTRACKING, 0, @tracking,0));
  CheckError(SystemParametersInfo(SPI_GETACTIVEWNDTRKTIMEOUT, 0, @timeout, 0));
  CheckError(SystemParametersInfo(SPI_GETACTIVEWNDTRKZORDER, 0, @zorder, 0));
  if IsWinArrSupported then
  begin
    CheckError(SystemParametersInfo(SPI_GETWINARRANGING, 0, @winarr, 0));
    winarrs := BStr(winarr);
  end else
    winarrs := 'not supported';

  MessageBox(GetDesktopWindow, PChar(
        'usage: xmouse [-SZsz] [delay]'#13#10+
        #13#10+
        '  delay'#9'active window tracking delay (milliseconds)'#13#10+
                 #9'  -1: tracking off'#13#10+
        '  -S'#9'disable Aero Snap'#13#10+
        '  -s'#9'enable Aero Snap'#13#10+
        '  -Z'#9'do not bring to top'#13#10+
        '  -z'#9'bring to top'#13#10+
        #13#10+
        #13#10+
        'current settings: '#13#10+
        #13#10+
        '  tracking'#9#9 + BStr(tracking) + #13#10 +
        '  delay'#9#9 + IStr(timeout) + ' msec.'#13#10 +
        '  bring to top'#9 + BStr(zorder) + #13#10 +
        '  Aero Snap'#9 + winarrs
      ), 'xmouse ' + Version, 0);
end;

var
  arg: string;
  i, j, code: Integer;
  timeout: Integer = -2;
  tracking: Boolean;
  winarr: Integer = -1;
  zorder: Integer = -1;
begin
  if ParamCount < 1 then
  begin
    Usage;
    Exit;
  end;

  for i := 1 to ParamCount do
  begin
    arg := ParamStr(i);
    if (arg[1] <> '-') or (arg = '-1') then
      Val(arg, timeout, code)
    else
      for j := 2 to Length(arg) do
        case arg[j] of
          'S': winarr := 0;
          's': winarr := 1;
          'Z': zorder := 0;
          'z': zorder := 1;
        end;
  end;

  if timeout > -2 then
  begin
    tracking := timeout > -1;
    CheckError(SystemParametersInfo(SPI_SETACTIVEWINDOWTRACKING, 0,
      Pointer(tracking), 0));
    if tracking then
      CheckError(SystemParametersInfo(SPI_SETACTIVEWNDTRKTIMEOUT, 0,
        Pointer(timeout), 0));
  end;

  if zorder > -1 then
    CheckError(SystemParametersInfo(SPI_SETACTIVEWNDTRKZORDER, 0,
      Pointer(zorder), 0));

  if winarr > -1 then
    CheckError(SystemParametersInfo(SPI_SETWINARRANGING, winarr, nil, 0));
end.
